#if !defined HAVE_COMP2COMB_H__
#define      HAVE_COMP2COMB_H__

#include "perm/reverse.h"
#include "fxttypes.h"

//: Conversion between combinations and compositions

//<<
// A composition P(n,k) of n into (at most) k parts corresponds to
// a combination C(N,K) of K=n parts from N=n+k-1 elements:
//   P(n, k)  <-->  C(N, K) == C(n+k-1, n)
//
// A combination C(N,K) of K elements out of N
// corresponds to a composition P(n,k) of n into (at most) k parts
// where k=N-K+1 and n=K:
//   C(N, K)  <-->  P(n, k) == P(K, N-K+1)
//>>

inline void comp2comb(const ulong *p, ulong k, ulong *c)
// Convert composition P(*, k) in p[] to combination in c[]
{
    for (ulong j=0,i=0,z=0; j<k; ++j)
    {
        ulong pj = p[j];
        for (ulong w=0; w<pj; ++w)   c[i++] = z++;
        ++z;
    }
}
// -------------------------

inline void comb2comp(const ulong *c, ulong N, ulong K, ulong *p)
// Convert combination C(N, K) in c[] to composition P(*,k) in p[]
// Must have: K>0
{
    ulong k = N-K+1;
    for (ulong z=0; z<k; ++z)  p[z] = 0;
    --k;
    ulong c1 = N;
    while ( K-- )
    {
        ulong c0 = c[K];
        ulong d = c1 - c0;
        k -= (d-1);
        ++p[k];
        c1 = c0;
    }
}
// -------------------------


inline void reverse_combination(ulong *c, ulong N, ulong K)
// Reverse order and complement values in combination c[]
// Equivalent to order reversion of the corresponding composition:
//   C <--> P  ==>  reverse_combination(C) <--> reverse(P)
{
    for (ulong z=0; z<K; ++z)  c[z] = N-1-c[z];
    reverse(c, K);
}
// -------------------------


#endif  // !defined HAVE_COMP2COMB_H__
